#! /usr/bin/perl -w

#this should take an input hmmsearch file, extract all hits from the file and realign them to a hmm file.output is a fasta format multiple alignment.

#-------------subs--------------

sub printinfo(){
    print"\n-------------------------------------------------\n";
    print" hmms2aln.pl info\n";
    print" This program is supposed to convert a hmmsearch file into a multiple alignment file\n";
    print"--------------------------------------\n";
    print"usage: perl hmms2aln.pl hmmsearch.file hmmprofile.file (opt. output_filename)\n";
    print" default output is to stdout\n";
    exit(0);
}
#end printinfo

#------------------------------

sub preparehmma($$){
    #get the sequence hit parts from the hmmsearch file and write to file
    my $infilename=shift;
    my $outfilename=shift;
    open (INFILE,"< $infilename") or print STDERR "unable to open $infilename\n" and return -1;
    open (OUTFILE,"> $outfilename") or print STDERR "unable to open $outfilename for writing\n" and return -1;
    #now read the sequences from infile
    my @seqnames=();
    my @seqs=();
    my $currseq="";
    my $tmpname="tmpname";
    my $seqcount=0;
    while ($inline=<INFILE>){
	#get the sequence name
	if($inline=~/(.+):\s+domain\s+(\d+)\s+.*,\s*from\s(\d+)\s*to\s*(\d+)\s*:.*(E\s*=\s*\S+).*/o){
	    $seqcount++;
	    my $name=$1;
	    my $domain=$2;
	    my $start=$3;
	    my $stop=$4;
	    my $evalue=$5;
	    #print STDERR "positive for $inline\n $name; $domain; $evalue\n";
	    if(length $currseq>0){
		$currseq=~s/[\.-]//g;
		push @seqs,$currseq;
	    }
	    if($domain==1){
		push @seqnames, "tmp".$seqcount."_delme "."$name ;$evalue pos:$start to $stop";
	    }else{
		#if this is not the only hit for this sequence
		$tmpname="tmp".$seqcount."_delme ".$domain."_"."$name ;$evalue pos:$start to $stop";
		push @seqnames,$tmpname;
	    }
	    if(length($1)>10){
		$tmpname=substr $1,0,10;
	    }else{
		$tmpname=$1;
	    }
	    $tmpname=~s/\|/\\\|/g;
	    $tmpname=~s/\+/\\\+/g;
	    #print STDERR "tmpname='$tmpname'\n";
	    $currseq="";
	    next;
	}
	if($inline=~/$tmpname\s+\d+\s+(.+)\s+\d+/){
	    $currseq=$currseq.$1;
	}else{
	    #print STDERR "line '$inline' not parseable for '$tmpname'\n";
	}
    }
    if(length $currseq>0){
	$currseq=~s/[\.-]//g;
	push @seqs,$currseq;
    }   
    #print STDERR "done reading $infilename:".scalar @seqnames." sequences\n";
    #end while reading from hmmsearch file
    #now write the sequences and names to a separate file
    my $counter=0;
    foreach $seqname(@seqnames){
	print OUTFILE ">$seqname\n";
	#print STDERR "printing for $seqname $counter of ".scalar @seqs."\n";
	print OUTFILE "$seqs[$counter]\n";
	$counter++;
    }
    #end writing to file
    close OUTFILE or print STDERR "Error on closing outfilehandle\n";
    close INFILE or print STDERR "error on closing infliehandle\n";
    return 0;
}
#end extract sequences from hmms and write to flatfile 

#--------------------------------------------------

sub reformat($$){
    #reformat the input file(stockholm format) to fasta format. and replace the names with those in the hmmdb file
    my $dbname=shift;
    my %nameshash=();
    open (INFILE,"<$dbname") or die "unable to open $dbname\n";
    while($inline=<INFILE>){
	chomp $inline;
	if($inline=~/>(\S+)/o){
	    $nameshash{$1}=substr $inline,1;
	}
    }
    close INFILE;
    my $inname=shift;
    my %seqhash=();
    #print "reformatting $inname to fasta\n";
    open (INFILE,"< $inname")or die "unable to open $inname for reading\n";
    my $startread=0;
    my $tmpname;
    my $tmpseq;
    my $counter=0;
    while ($inline=<INFILE>){
	if($inline=~/\#\s*stockholm/i){
	    $startread=1;
	    next;
	}
	if(($inline=~/\#/)||($inline!~/\S+/)){
	    next;
	}elsif($startread==1){
	    if($inline=~/^\/\//){
		#if the stockholm alignment is closed by '//' 
		$startread=0;
		last;
	    }
	    if($inline=~/(\S+)\s+(\S+)/){
		$tmpname=$1;
		$tmpseq=$2;
		if(defined $nameshash{$tmpname}){
		    $tmpname=$nameshash{$tmpname};
		}
		if(defined $seqhash{$tmpname}){
		    $seqhash{$tmpname}.=$tmpseq;
		}else{
		    $seqhash{$tmpname}=$tmpseq;
		    $counter++;
		}
	    }
	}
    }
    if($startread==1){
	print "Stockholm alignment file $inname not closed by '//'\n";
    }
    #end while reading from infile
    #print "$counter sequences\n";
    close INFILE;
    if($writetofile==1){
	open (OUTFILE,"> $outfilename");
	@seqnames=keys %seqhash;
	foreach $seqname(@seqnames){
	    if($seqname=~/^tmp\d+_delme\s+(.+)/o){
		print OUTFILE ">$1\n";
	    }else{
		print OUTFILE ">$seqname\n";
	    }
	    $tmpseq=$seqhash{$seqname};
	    $tmpseq=~s/\./-/g;
	    print OUTFILE "$tmpseq\n";
	}
	close OUTFILE;
    }else{
	@seqnames=keys %seqhash;
	foreach $seqname(@seqnames){
	    if($seqname=~/^tmp\d+_delme\s+(.+)/o){
		print ">$1\n";
	    }else{
		print ">$seqname\n";
	    }
	    $tmpseq=$seqhash{$seqname};
	    $tmpseq=~s/\./-/g;
	    print "$tmpseq\n";
	}
    }
   # print "done reformatting\n";
}
#end sub reformat

#--------------------main---------------------


if((scalar @ARGV)==2){
    #output=stdout
    $hmmsfile=$ARGV[0];
    $hmmfile=$ARGV[1];
    $writetofile=0;
}elsif((scalar @ARGV)==3){
    $hmmsfile=$ARGV[0];
    $hmmfile=$ARGV[1];
    $writetofile=1;
    $outfilename=$ARGV[2];
}else{
    printinfo();
}


$basename="tmpbasename";
if($hmmsfile=~/(.+?)\./){
    $basename=$1;
}
$hmmdb="$basename.hmmdb";
$hmmaname="$basename.hmmtmp";
#print "doing hmmalign\n";
#to do this I need to get all sequences from the hmmsearch file and save them to a separate file $hmmdbname.
if((preparehmma($hmmsfile,$hmmdb))==-1){
    #if sth went wrong
    print STDERR "unable to do hmmalign\n";
}else{
    $command="hmmalign $hmmfile $hmmdb >$hmmaname";
    system($command)==0 or print "unable to do '$command'\n";
    reformat($hmmdb,$hmmaname);
}
#remove the temporary files that are not needed
unlink $hmmdb;
unlink $hmmaname;
